import time

from croterline.context import Context
from croterline.process import SubProcess, get_ctx


def p_func(*args, **kwargs):
    print("Args", args)
    print("Kwargs", kwargs)
    i = 0
    ctx = get_ctx()
    while True:
        ctx.sub_chan << "Running" << "SubProcess"
        print("Recv from main: ", str << ctx.main_chan)
        i += 1
        if i == 5:
            ctx.sub_chan << "end"


def p_func2(*args, **kwargs):
    print("args: ", args)
    print("kwargs: ", kwargs)
    raise Exception("Test")


class TestSubProcess:
    def test_run(self):
        print("start")
        sp = SubProcess("test", p_func, Context(), 1, 2, 3, k1=1, k2=2)
        sp.start()

        while True:
            s = str << sp.ctx.sub_chan
            sp.ctx.main_chan << "Resp"
            print("Recv from sub: ", s)
            if s == "end":
                break
        print("finished")
        sp.terminate()

    def test_input(self):
        print("test_input")
        sp = SubProcess("test2", p_func2, Context(), 1, 2, 3, host=1, port=2)
        sp.start()



from typing import Callable, Any

from magicoca.chan import Chan


class Context:
    def __init__(self):
        """
        main_chan: Chan[Any] = main to sub
        sub_chan: Chan[Any] = sub to main
        哪个进程发送用哪个
        """
        self.main_chan: Chan[Any] = Chan[Any]()  # main to sub
        self.sub_chan: Chan[Any] = Chan[Any]()  # sub to main

    def set_value(self, key: str, value: Any):
        setattr(self, key, value)

    def get_value(self, key: str) -> Any:
        return getattr(self, key)



from multiprocessing import Process as _Process
from typing import Callable, Any

from mypy.nodes import TypeAlias

from croterline.context import Context
from croterline.utils import IsMainProcess

ProcessFuncType: TypeAlias = Callable[[tuple[Any, ...], dict[str, Any]], None]

_current_ctx: "Context | None" = None  # 注入当前进程上下文


class SubProcess:
    def __init__(
        self,
        name: str,
        func: ProcessFuncType,
        ctx: Context = Context(),
        *args,
        **kwargs,
    ):
        self.name = name
        self.func = func
        self.ctx = ctx
        self.args = args
        self.kwargs = kwargs
        self.process: _Process | None = None

    def start(self):
        self.process = _Process(
            target=_wrapper,
            args=(self.func, self.ctx, *self.args),
            kwargs=self.kwargs,
        )
        self.process.start()

    def terminate(self):
        self.process.terminate()

    def join(self, timeout: float = 0):
        self.process.join(timeout=timeout)


def get_ctx() -> Context | None:
    """
    获取进程上下文，在主进程中调用需指定进程名称，若在子进程中调用则无需指定进程名称
    Returns:
        进程上下文
    """
    if IsMainProcess:
        raise RuntimeError(
            "get_ctx without name can only be called in the sub process."
        )
    return _current_ctx


def _wrapper(func: ProcessFuncType, ctx: Context, *args, **kwargs):
    global _current_ctx

    _current_ctx = ctx
    func(*args, **kwargs)
